'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator;

function _load_regenerator() {
  return _regenerator = _interopRequireDefault(require('babel-runtime/regenerator'));
}

var _bluebird;

function _load_bluebird() {
  return _bluebird = require('bluebird');
}

var _consumer;

function _load_consumer() {
  return _consumer = require('../../../consumer');
}

var _bitId;

function _load_bitId() {
  return _bitId = require('../../../bit-id');
}

var _untagComponent;

function _load_untagComponent() {
  return _untagComponent = require('../../../scope/component-ops/untag-component');
}

var _hasWildcard;

function _load_hasWildcard() {
  return _hasWildcard = _interopRequireDefault(require('../../../utils/string/has-wildcard'));
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * in case the untagged version is the current version in bitmap, update to the latest version
 * or, remove the version completely in case all versions were untagged
 */
function updateBitMap(consumer, untagResults) {
  untagResults.forEach(function (result) {
    var id = result.id,
        versions = result.versions,
        component = result.component;

    var currentId = consumer.bitMap.getBitId(id, { ignoreVersion: true });
    if (currentId.hasVersion() && versions.includes(currentId.version)) {
      var newVersion = component.versionArray.length ? component.latest() : null;
      var newId = currentId.changeVersion(newVersion);
      consumer.bitMap.updateComponentId(newId);
    }
  });
}

exports.default = function () {
  var _ref = (0, (_bluebird || _load_bluebird()).coroutine)( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee2(version, force, id) {
    var _this = this;

    var consumer, untag, results;
    return (_regenerator || _load_regenerator()).default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _context2.next = 2;
            return (0, (_consumer || _load_consumer()).loadConsumer)();

          case 2:
            consumer = _context2.sent;

            untag = function () {
              var _ref2 = (0, (_bluebird || _load_bluebird()).coroutine)( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee() {
                var idHasWildcard, bitId, result;
                return (_regenerator || _load_regenerator()).default.wrap(function _callee$(_context) {
                  while (1) {
                    switch (_context.prev = _context.next) {
                      case 0:
                        idHasWildcard = (0, (_hasWildcard || _load_hasWildcard()).default)(id);

                        if (!idHasWildcard) {
                          _context.next = 3;
                          break;
                        }

                        return _context.abrupt('return', (0, (_untagComponent || _load_untagComponent()).removeLocalVersionsForComponentsMatchedByWildcard)(consumer.scope, version, force, id));

                      case 3:
                        if (!id) {
                          _context.next = 10;
                          break;
                        }

                        bitId = consumer.getParsedId(id);
                        // a user might run the command `bit untag id@version` instead of `bit untag id version`

                        if (bitId.hasVersion() && !version) version = bitId.version;
                        _context.next = 8;
                        return (0, (_untagComponent || _load_untagComponent()).removeLocalVersion)(consumer.scope, bitId, version, force);

                      case 8:
                        result = _context.sent;
                        return _context.abrupt('return', [result]);

                      case 10:
                        return _context.abrupt('return', (0, (_untagComponent || _load_untagComponent()).removeLocalVersionsForAllComponents)(consumer.scope, version, force));

                      case 11:
                      case 'end':
                        return _context.stop();
                    }
                  }
                }, _callee, _this);
              }));

              return function untag() {
                return _ref2.apply(this, arguments);
              };
            }();

            _context2.next = 6;
            return untag();

          case 6:
            results = _context2.sent;
            _context2.next = 9;
            return consumer.scope.objects.persist();

          case 9:
            updateBitMap(consumer, results);
            _context2.next = 12;
            return consumer.onDestroy();

          case 12:
            return _context2.abrupt('return', results);

          case 13:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, this);
  }));

  function unTagAction(_x, _x2, _x3) {
    return _ref.apply(this, arguments);
  }

  return unTagAction;
}();